@@ -4,7 +4,11 @@ import java.util.ArrayList;
 
 import org.gcube.portlets.user.td.columnwidget.client.dimension.CodelistSelectionDialog;
 import org.gcube.portlets.user.td.columnwidget.client.dimension.CodelistSelectionListener;
+import org.gcube.portlets.user.td.columnwidget.client.mapping.ColumnMappingDialog;
+import org.gcube.portlets.user.td.columnwidget.client.mapping.ColumnMappingListProperties;
+import org.gcube.portlets.user.td.columnwidget.client.mapping.ColumnMappingListener;
 import org.gcube.portlets.user.td.columnwidget.client.progress.ChangeColumnTypeProgressDialog;
+import org.gcube.portlets.user.td.columnwidget.client.progress.ProgressDialogListener;
 import org.gcube.portlets.user.td.columnwidget.client.properties.ColumnDataPropertiesCombo;
 import org.gcube.portlets.user.td.columnwidget.client.properties.ColumnDataTypeProperties;
 import org.gcube.portlets.user.td.columnwidget.client.properties.ColumnTypeCodeProperties;
@@ -23,8 +27,12 @@ import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
 import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.TabResource;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.column.mapping.ColumnMappingList;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.column.type.ChangeColumnTypeSession;
+import org.gcube.portlets.user.td.widgetcommonevent.client.event.ChangeTableRequestEvent;
 import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
+import org.gcube.portlets.user.td.widgetcommonevent.client.type.ChangeTableRequestType;
+import org.gcube.portlets.user.td.widgetcommonevent.client.type.ChangeTableWhy;
 import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnDataType;
@@ -73,7 +81,7 @@ import com.sencha.gxt.widget.core.client.info.Info;
  * 
  */
 public class ChangeColumnTypePanel extends FramedPanel implements
-		CodelistSelectionListener {
+		CodelistSelectionListener, ColumnMappingListener,  ProgressDialogListener{
 	protected String WIDTH = "640px";
 	protected String HEIGHT = "520px";
 	protected EventBus eventBus;
@@ -93,11 +101,16 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 
 	protected ComboBox<TabResource> comboDimensionType = null;
 	protected FieldLabel comboDimensionTypeLabel;
+	protected ListStore<TabResource> storeComboDimensionType;
 
 	protected ComboBox<ColumnData> comboColumnReferenceType = null;
 	protected FieldLabel comboColumnReferenceTypeLabel;
 	protected ListStore<ColumnData> storeComboColumnReferenceType;
 
+	protected ComboBox<ColumnMappingList> comboColumnMapping = null;
+	protected FieldLabel comboColumnMappingLabel;
+	protected ListStore<ColumnMappingList> storeComboColumnMapping;
+
 	protected ComboBox<TimeDimensionTypeElement> comboTimeDimensionType = null;
 	protected FieldLabel comboTimeDimensionTypeLabel;
 
@@ -273,7 +286,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 		// comboDimensionType
 		TabResourceProperties propsDimensionType = GWT
 				.create(TabResourceProperties.class);
-		ListStore<TabResource> storeComboDimensionType = new ListStore<TabResource>(
+		storeComboDimensionType = new ListStore<TabResource>(
 				propsDimensionType.id());
 
 		comboDimensionType = new ComboBox<TabResource>(storeComboDimensionType,
@@ -309,6 +322,26 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 		comboColumnReferenceTypeLabel = new FieldLabel(
 				comboColumnReferenceType, "Column");
 
+		// TODO
+		// ColumnMapping
+		ColumnMappingListProperties propsColumnMapping = GWT
+				.create(ColumnMappingListProperties.class);
+		storeComboColumnMapping = new ListStore<ColumnMappingList>(
+				propsColumnMapping.id());
+
+		comboColumnMapping = new ComboBox<ColumnMappingList>(
+				storeComboColumnMapping, propsColumnMapping.name());
+		Log.trace("ComboColumnMapping created");
+
+		addHandlersForComboColumnMapping(propsColumnMapping.name());
+
+		comboColumnMapping.setEmptyText("Select optional mapping...");
+		comboColumnMapping.setWidth(191);
+		comboColumnMapping.setEditable(false);
+		comboColumnMapping.setTriggerAction(TriggerAction.ALL);
+
+		comboColumnMappingLabel = new FieldLabel(comboColumnMapping, "Mapping");
+
 		// comboTimeDimensionType
 		TimeDimensionTypeProperties propsTimeDimensionType = GWT
 				.create(TimeDimensionTypeProperties.class);
@@ -364,6 +397,8 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 				new Margins(1)));
 		v.add(comboColumnReferenceTypeLabel, new VerticalLayoutData(1, -1,
 				new Margins(1)));
+		/*v.add(comboColumnMappingLabel, new VerticalLayoutData(1, -1,
+				new Margins(1)));*/
 		v.add(comboTimeDimensionTypeLabel, new VerticalLayoutData(1, -1,
 				new Margins(1)));
 		v.add(change, new VerticalLayoutData(-1, -1, new Margins(10, 0, 10, 0)));
@@ -373,6 +408,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 		comboAttributeTypeLabel.setVisible(false);
 		comboDimensionTypeLabel.setVisible(false);
 		comboColumnReferenceTypeLabel.setVisible(false);
+		comboColumnMappingLabel.setVisible(false);
 		comboTimeDimensionTypeLabel.setVisible(false);
 		comboLocaleTypeLabel.setVisible(false);
 
@@ -396,7 +432,8 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 
 		});
 	}
-
+	
+	//TODO
 	protected void addHandlersForComboColumnTypeCode(
 			final LabelProvider<ColumnTypeCodeElement> labelProvider) {
 		comboColumnTypeCode
@@ -520,6 +557,15 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 														+ "!"));
 						Log.debug("ComboColumnReferenceType selected: "
 								+ event.getSelectedItem());
+						
+						//TODO Mapping fix
+						/*
+						comboColumnMapping.reset();
+						storeComboColumnMapping.clear();
+						storeComboColumnMapping.commitChanges();
+						comboColumnMappingLabel.setVisible(true);
+						forceLayout();
+						*/
 
 					}
 
@@ -527,6 +573,23 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 
 	}
 
+	protected void addHandlersForComboColumnMapping(
+			final LabelProvider<ColumnMappingList> labelProvider) {
+
+		comboColumnMapping.addTriggerClickHandler(new TriggerClickHandler() {
+
+			@Override
+			public void onTriggerClick(TriggerClickEvent event) {
+				Log.debug("ComboColumnMapping TriggerClickEvent");
+				callColumnMappingDialog();
+				comboColumnMapping.collapse();
+
+			}
+
+		});
+
+	}
+
 	protected void addHandlersForComboTimeDimensionType(
 			final LabelProvider<TimeDimensionTypeElement> labelProvider) {
 		comboTimeDimensionType
@@ -550,6 +613,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 				});
 	}
 
+	//TODO
 	protected void updateColumnType(ColumnTypeCode type) {
 		Log.debug("Update ColumnTypeCode " + type.toString());
 		switch (type) {
@@ -559,6 +623,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(false);
 			comboDimensionTypeLabel.setVisible(false);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(false);
 			break;
 		case ATTRIBUTE:
@@ -567,6 +632,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(true);
 			comboDimensionTypeLabel.setVisible(false);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(false);
 			break;
 		case DIMENSION:
@@ -575,6 +641,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(false);
 			comboDimensionTypeLabel.setVisible(true);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(false);
 			break;
 		case MEASURE:
@@ -583,6 +650,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(false);
 			comboDimensionTypeLabel.setVisible(false);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(false);
 			break;
 		case TIMEDIMENSION:
@@ -591,6 +659,7 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(false);
 			comboDimensionTypeLabel.setVisible(false);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(true);
 			break;
 		default:
@@ -599,10 +668,15 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 			comboAttributeTypeLabel.setVisible(false);
 			comboDimensionTypeLabel.setVisible(false);
 			comboColumnReferenceTypeLabel.setVisible(false);
+			comboColumnMappingLabel.setVisible(false);
 			comboTimeDimensionTypeLabel.setVisible(false);
 			break;
 		}
-
+		
+		//Reset comboDimensionType
+		comboDimensionType.reset();
+		comboDimensionType.clear();
+		storeComboDimensionType.commitChanges();
 		forceLayout();
 	}
 
@@ -796,6 +870,12 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 					ColumnData columnReference = comboColumnReferenceType
 							.getCurrentValue();
 					if (columnReference != null) {
+						ColumnMappingList mapping = comboColumnMapping
+								.getValue();
+						if (mapping != null) {
+							changeColumnTypeSession
+									.setColumnMappingList(mapping);
+						}
 						changeColumnTypeSession
 								.setCodelistColumnReference(columnReference);
 						callChangeColumnType();
@@ -855,8 +935,9 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 					}
 
 					public void onSuccess(Void result) {
-						changeColumnTypeProgressDialog = new ChangeColumnTypeProgressDialog(
-								changeColumnTypeSession, eventBus);
+						callChangeColumnTypeProgressDialog();
+						
+						
 					}
 
 				});
@@ -873,7 +954,8 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 	@Override
 	public void selected(TabResource tabResource) {
 		Log.debug("Selected Codelist: " + tabResource);
-		comboDimensionType.setValue(tabResource);
+		comboDimensionType.setValue(tabResource, true);
+		
 		retrieveColumnData(tabResource);
 	}
 
@@ -964,4 +1046,94 @@ public class ChangeColumnTypePanel extends FramedPanel implements
 		}
 	}
 
+	// TODO
+	protected void callColumnMappingDialog() {
+		ColumnData selectedColumn = comboColumn.getCurrentValue();
+		if (selectedColumn == null) {
+			UtilsGXT3.alert("Attention",
+					"Column not selected!");
+			return;
+		}
+
+		TabResource dimensionTR = comboDimensionType.getValue();
+		if (dimensionTR == null) {
+			UtilsGXT3.alert("Attention",
+					"Codelist not selected!");
+			return;
+		}
+
+		ColumnData referenceColumn = comboColumnReferenceType.getValue();
+		if (referenceColumn == null) {
+			UtilsGXT3.alert("Attention",
+					"Reference column not selected!");
+			return;
+		}
+
+		ColumnMappingDialog columnMappingDialog = new ColumnMappingDialog(trId,
+				selectedColumn, dimensionTR, referenceColumn, eventBus);
+		columnMappingDialog.addColumnMappingListener(this);
+		columnMappingDialog.show();
+	}
+
+	@Override
+	public void selectedColumnMapping(ColumnMappingList columnMappingList) {
+		Log.debug("Selected ColumnMapping: " + columnMappingList);
+		comboColumnMapping.setValue(columnMappingList, true);
+
+	}
+
+	@Override
+	public void abortedColumnMapping() {
+		Log.debug("Column Mapping selection Aborted");
+
+	}
+
+	@Override
+	public void failedColumnMapping(String reason, String detail) {
+		Log.error("Error selecting Column Mapping:" + reason + " " + detail);
+
+	}
+
+	
+	public void close() {
+		if (parent != null) {
+			parent.close();
+		}
+	}
+	
+	
+	protected void callChangeColumnTypeProgressDialog(){
+		changeColumnTypeProgressDialog = new ChangeColumnTypeProgressDialog(
+				eventBus);
+		changeColumnTypeProgressDialog.addProgressDialogListener(this);
+		changeColumnTypeProgressDialog.show();
+	} 
+
+	@Override
+	public void operationComplete(TRId trId) {
+		ChangeTableWhy why = ChangeTableWhy.TABLEUPDATED;
+		ChangeTableRequestEvent changeTableRequestEvent = new ChangeTableRequestEvent(
+				ChangeTableRequestType.CHANGECOLUMNTYPE, trId, why);
+		eventBus.fireEvent(changeTableRequestEvent);
+		close();
+
+	}
+
+	@Override
+	public void operationFailed(Throwable caught, String reason) {
+		UtilsGXT3.alert("Error in Change Column Type", reason);
+		close();
+	}
+
+	@Override
+	public void operationStopped(TRId trId, String reason, String details) {
+		ChangeTableWhy why = ChangeTableWhy.TABLECURATION;
+		ChangeTableRequestEvent changeTableRequestEvent = new ChangeTableRequestEvent(
+				ChangeTableRequestType.CHANGECOLUMNTYPE, trId, why);
+		eventBus.fireEvent(changeTableRequestEvent);
+		close();
+	}
+
+	
+
 }
