@@ -16,7 +16,9 @@ import org.gcube.portlets.user.td.columnwidget.client.utils.UtilsGXT3;
 import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
 import org.gcube.portlets.user.td.gwtservice.shared.Constants;
 import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
+import org.gcube.portlets.user.td.gwtservice.shared.task.InvocationS;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.ConditionCode;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.DimensionRow;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.RefColumn;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.batch.Occurrences;
@@ -100,6 +102,8 @@ public class ReplaceBatchPanel extends FramedPanel implements
 
 	protected RefColumn refColumn;
 	protected String columnId;
+	protected ConditionCode conditionCode; // For Curation
+	protected String validationColumnColumnId; // For Curation
 	protected RequestProperties requestProperties;
 	protected boolean curation;
 	protected ColumnData column;
@@ -129,7 +133,7 @@ public class ReplaceBatchPanel extends FramedPanel implements
 	protected Grid<ReplaceEntry> grid;
 	protected ListStore<ReplaceEntry> store;
 	protected HTML info;
-	
+
 	private boolean simpleReplace;
 
 	public ReplaceBatchPanel(ReplaceBatchDialog parent, TRId trId,
@@ -139,10 +143,18 @@ public class ReplaceBatchPanel extends FramedPanel implements
 		this.trId = trId;
 		this.requestProperties = requestProperties;
 		this.eventBus = eventBus;
-		columnId = (String) requestProperties.getMap().get(
-				Constants.REQUEST_PROPERTIES_COLUMNID);
-		refColumn = (RefColumn) requestProperties.getMap().get(
-				Constants.REQUEST_PROPERTIES_REFCOLUMN);
+
+		InvocationS invocationS = (InvocationS) requestProperties.getMap().get(
+				Constants.REQUEST_PROPERTIES_INVOCATIONS);
+
+		columnId = invocationS.getColumnId();
+
+		refColumn = invocationS.getRefColumn();
+
+		conditionCode = (ConditionCode) requestProperties.getMap().get(
+				Constants.REQUEST_PROPERTIES_CONDITIONCODE);
+		validationColumnColumnId = (String) requestProperties.getMap().get(
+				Constants.REQUEST_PROPERTIES_VALIDATIONCOLUMNCOLUMNID);
 
 		connection = null;
 		hasValidationColumns = false;
@@ -160,6 +172,9 @@ public class ReplaceBatchPanel extends FramedPanel implements
 		this.trId = trId;
 		this.columnName = columnName;
 		this.eventBus = eventBus;
+		this.conditionCode = null;
+		this.validationColumnColumnId = null;
+
 		connection = null;
 		hasValidationColumns = false;
 		Log.debug("Create BatchReplacePanel(): [" + trId.toString()
@@ -563,7 +578,8 @@ public class ReplaceBatchPanel extends FramedPanel implements
 			final AsyncCallback<ListLoadResult<ReplaceEntry>> callback) {
 
 		OccurrencesForReplaceBatchColumnSession occurrencesSession = new OccurrencesForReplaceBatchColumnSession(
-				column, showOccurencesType, hasValidationColumns);
+				column, showOccurencesType, hasValidationColumns,
+				conditionCode, validationColumnColumnId);
 
 		TDGWTServiceAsync.INSTANCE.getOccurrencesForBatchReplace(
 				occurrencesSession,
@@ -638,15 +654,73 @@ public class ReplaceBatchPanel extends FramedPanel implements
 
 	}
 
+	//
 	protected ReplaceEntry getRecord(Occurrences entry) {
 		ReplaceEntry data = null;
-		if (column.isViewColumn()) {
-			data = new ReplaceEntry(entry.getValue(), entry.getRowId(),
-					entry.getNumber(), null, null);
+		if (conditionCode == null) {
+			if (column.isViewColumn()) {
+				data = new ReplaceEntry(entry.getValue(), entry.getRowId(),
+						entry.getNumber(), null, null);
+			} else {
+				data = new ReplaceEntry(entry.getValue(), entry.getNumber(),
+						null, null);
+			}
 		} else {
-			data = new ReplaceEntry(entry.getValue(), entry.getNumber(), null,
-					null);
+			switch (conditionCode) {
+			case AllowedColumnType:
+				break;
+			case AmbiguousValueOnExternalReference:
+				data = new ReplaceEntry(entry.getValue(), entry.getRowId(),
+						entry.getNumber(), null, null);
+				break;
+			case CastValidation:
+				break;
+			case CodeNamePresence:
+				break;
+			case DuplicateTupleValidation:
+				break;
+			case DuplicateValueInColumn:
+				break;
+			case GenericTupleValidity:
+				break;
+			case GenericValidity:
+				break;
+			case MaxOneCodenameForDataLocale:
+				break;
+			case MissingValueOnExternalReference:
+				if (column.isViewColumn()) {
+					data = new ReplaceEntry(entry.getValue(), entry.getRowId(),
+							entry.getNumber(), null, null);
+				} else {
+					data = new ReplaceEntry(entry.getValue(),
+							entry.getNumber(), null, null);
+				}
+				break;
+			case MustContainAtLeastOneDimension:
+				break;
+			case MustContainAtLeastOneMeasure:
+				break;
+			case MustHaveDataLocaleMetadataAndAtLeastOneLabel:
+				break;
+			case OnlyOneCodeColumn:
+				break;
+			case OnlyOneCodenameColumn:
+				break;
+			case ValidPeriodFormat:
+				break;
+			default:
+				if (column.isViewColumn()) {
+					data = new ReplaceEntry(entry.getValue(), entry.getRowId(),
+							entry.getNumber(), null, null);
+				} else {
+					data = new ReplaceEntry(entry.getValue(),
+							entry.getNumber(), null, null);
+				}
+				break;
+
+			}
 		}
+
 		return data;
 	}
 
@@ -665,10 +739,10 @@ public class ReplaceBatchPanel extends FramedPanel implements
 			}
 		} else {
 			Log.debug("In curation from validation");
-			if(connection!=null){
+			if (connection != null) {
 				callDimensionRowSelectionDialog();
 			} else {
-				
+
 			}
 		}
 	}
@@ -694,7 +768,8 @@ public class ReplaceBatchPanel extends FramedPanel implements
 	protected void startReplaceBatch(
 			ArrayList<ReplaceEntry> effectiveReplaceList) {
 		ReplaceBatchColumnSession replaceBatchColumnSession = new ReplaceBatchColumnSession(
-				trId, column, effectiveReplaceList, column.isViewColumn());
+				trId, column, effectiveReplaceList, column.isViewColumn(),
+				connection);
 		parent.startBatchReplace(replaceBatchColumnSession);
 	}
 
@@ -727,32 +802,87 @@ public class ReplaceBatchPanel extends FramedPanel implements
 	protected void callDimensionRowSelectionDialog() {
 		simpleReplace = false;
 		Log.debug("callDimensionRowSelectionDialog");
-		CellData cellData=null;
-		ColumnData col=null;
-		boolean workOnTable=false;
-		if(curation){
-			if(connection!=null){
+		CellData cellData = null;
+		ColumnData col = null;
+		boolean workOnTable = false;
+		if (curation) {
+			if (connection != null) {
 				cellData = new CellData(currentReplaceEntry.getValue(),
-					connection.getName(), connection.getColumnId(), connection.getLabel(),
-					currentReplaceEntry.getRowId(), 0, 0);
-				col=connection;
-				workOnTable=true;
+						connection.getName(), connection.getColumnId(),
+						connection.getLabel(), currentReplaceEntry.getRowId(),
+						0, 0);
+				col = connection;
+				workOnTable = true;
+
 			} else {
 				Log.error("No connection retrieved");
 			}
+			Log.debug("Col: " + col + ", CellData:" + cellData);
+			DimensionRowSelectionDialog dialogDimensionRowSelection;
+			
+			switch (conditionCode){
+			case AllowedColumnType:
+				break;
+			case AmbiguousValueOnExternalReference:
+				dialogDimensionRowSelection = new DimensionRowSelectionDialog(
+						col, cellData, workOnTable, true, true, true, false, eventBus);
+				dialogDimensionRowSelection.addListener(this);
+				dialogDimensionRowSelection.show();
+				break;
+			case CastValidation:
+				break;
+			case CodeNamePresence:
+				break;
+			case DuplicateTupleValidation:
+				break;
+			case DuplicateValueInColumn:
+				break;
+			case GenericTupleValidity:
+				break;
+			case GenericValidity:
+				break;
+			case MaxOneCodenameForDataLocale:
+				break;
+			case MissingValueOnExternalReference:
+				dialogDimensionRowSelection = new DimensionRowSelectionDialog(
+						col, cellData, workOnTable, eventBus);
+				dialogDimensionRowSelection.addListener(this);
+				dialogDimensionRowSelection.show();
+				break;
+			case MustContainAtLeastOneDimension:
+				break;
+			case MustContainAtLeastOneMeasure:
+				break;
+			case MustHaveDataLocaleMetadataAndAtLeastOneLabel:
+				break;
+			case OnlyOneCodeColumn:
+				break;
+			case OnlyOneCodenameColumn:
+				break;
+			case ValidPeriodFormat:
+				break;
+			default:
+				dialogDimensionRowSelection = new DimensionRowSelectionDialog(
+						col, cellData, workOnTable, eventBus);
+				dialogDimensionRowSelection.addListener(this);
+				dialogDimensionRowSelection.show();
+				break;
+			
+			}
+			
 		} else {
 			cellData = new CellData(currentReplaceEntry.getValue(),
-				column.getName(), column.getColumnId(), column.getLabel(),
-				currentReplaceEntry.getRowId(), 0, 0);
-			col=column;
-			workOnTable=false;
-		}
-		Log.debug("Col: "+col+", CellData:"+cellData);
-		DimensionRowSelectionDialog dialogDimensionRowSelection = new DimensionRowSelectionDialog(
-				col, cellData, workOnTable,eventBus);
-		dialogDimensionRowSelection.addListener(this);
-		dialogDimensionRowSelection.show();
+					column.getName(), column.getColumnId(), column.getLabel(),
+					currentReplaceEntry.getRowId(), 0, 0);
+			col = column;
+			workOnTable = false;
 
+			Log.debug("Col: " + col + ", CellData:" + cellData);
+			DimensionRowSelectionDialog dialogDimensionRowSelection = new DimensionRowSelectionDialog(
+					col, cellData, workOnTable, eventBus);
+			dialogDimensionRowSelection.addListener(this);
+			dialogDimensionRowSelection.show();
+		}
 	}
 
 	@Override
